@@ -34,10 +34,11 @@
 #' @param N number of units to draw. Defaults to the length of the vector of
 #' probabilities or latent data you provided.
 #' @param link link function between the latent variable and the probability of
-#' a postiive outcome, e.g. "logit", "probit", or "identity". For the "identity"
+#' a positive outcome, e.g. "logit", "probit", or "identity". For the "identity"
 #' link, the latent variable must be a probability.
 #' @param latent If the user provides a link argument other than identity, they
 #' should provide the variable `latent` rather than `prob` or `mean`
+#' @param strict Logical indicating whether values outside the provided breaks should be coded as NA. Defaults to \code{FALSE}, in which case effectively additional breaks are added between -Inf and the lowest break and between the highest break and Inf.
 #' @param quantile_y A vector of quantiles; if provided, rather than drawing
 #' stochastically from the distribution of interest, data will be drawn at
 #' exactly those quantiles.
@@ -201,12 +202,6 @@ draw_categorical <- function(prob = link(latent), N = NULL,
         )
       }
       prob <- matrix(rep(prob, N), byrow = TRUE, ncol = length(prob), nrow = N)
-      message(
-        "For a categorical (multinomial) distribution, a matrix of ",
-        "probabilities should be provided. The data below is generated by ",
-        "interpreting the vector of category probabilities you provided as ",
-        "identical for each observation."
-      )
     } else {
       stop(
         "For a categorical (multinomial) distribution, a matrix of ",
@@ -214,7 +209,7 @@ draw_categorical <- function(prob = link(latent), N = NULL,
       )
     }
   }
-  if (!all(apply(prob, 1, min) > 0)) {
+  if (any(prob < 0)) {
     stop(
       "For a categorical (multinomial) distribution, the elements of `prob` ",
       "should be positive and sum to a positive number."
@@ -250,10 +245,13 @@ draw_categorical <- function(prob = link(latent), N = NULL,
 
 #' @rdname draw_discrete
 #' @export
-draw_ordered <- function(x = link(latent), breaks = c(-1, 0, 1),
+draw_ordered <- function(x = link(latent),
+                         breaks = c(-1, 0, 1),
                          break_labels = NULL,
-                         N = length(x), latent = NULL,
-                         link = "identity", quantile_y = NULL) {
+                         N = length(x),
+                         latent = NULL,
+                         strict = FALSE,
+                         link = "identity") {
 
   # Handle link function - try matching normal way, and fallback
   # to manual logic for probit/logit
@@ -282,37 +280,35 @@ draw_ordered <- function(x = link(latent), breaks = c(-1, 0, 1),
     stop("`N` must be an even multiple of the length of `x`.")
   }
 
-  # Pre-pend -Inf
-  if (any(breaks[1] > x)) {
-    breaks <- c(-Inf, breaks)
-  }
-  # Post-pend Inf
-  if (any(breaks[length(breaks)] < x)) {
-    breaks <- c(breaks, Inf)
-  }
+  add_to_value <- !strict & !any(is.infinite(breaks) & sign(breaks))
 
   # Make sure break labels are concordant with breaks.
   if (!is.null(break_labels) &&
     (is.vector(break_labels) &&
       !is.logical(break_labels) &&
       all(!is.na(break_labels)) &&
-      length(break_labels) != length(breaks) - 1)) {
+      length(break_labels) != length(breaks) + ifelse(add_to_value, 1, -1))) {
     stop(
-      "Break labels should be of length one less than breaks. ",
+      "Break labels should be of length one more than breaks. ",
       "Currently you have ", length(break_labels), " bucket labels and ",
       length(breaks) - 1, " buckets of data."
     )
   }
 
   # Output
+
   if (!is.null(break_labels)) {
-    return(factor(
-      break_labels[findInterval(x, breaks)],
+    ret <- factor(
+      break_labels[findInterval(x, breaks) + add_to_value],
       levels = break_labels
-    ))
+    )
   } else {
-    return(findInterval(x, breaks))
+    ret <- findInterval(x, breaks) + add_to_value
   }
+  if(strict == TRUE){
+    ret[x > breaks[length(breaks)] | x < breaks[1]] <- NA
+  }
+  return(ret)
 }
 
 #' @rdname draw_discrete
@@ -368,14 +364,14 @@ draw_binary <- function(prob = link(latent), N = length(prob),
   # to manual logic for probit/logit
   link <- tryCatch(match.fun(link), error = handle_link_functions(link))
 
-  return(draw_binomial(
+  draw_binomial(
     prob = prob,
     N = N,
     link = link,
     trials = 1,
     latent = latent,
     quantile_y = quantile_y
-  ))
+  )
 }
 
 #' @rdname draw_discrete
@@ -386,7 +382,7 @@ draw_likert <- function(x,
                         N = length(x),
                         latent = NULL,
                         link = "identity",
-                        quantile_y = NULL) {
+                        strict = !is.null(breaks)) {
   if (is.null(breaks) && is.null(type)) {
     stop("You must provide either `breaks` or `type` to a `draw_likert()` ",
          "call.")
@@ -436,44 +432,28 @@ draw_likert <- function(x,
     )
   }
 
-  return(draw_ordered(
+  draw_ordered(
     x = x,
     breaks = breaks,
     N = N,
     link = link,
     latent = latent,
     break_labels = break_labels,
-    quantile_y = quantile_y
-  ))
+    strict = strict
+  )
 }
 
 #' @rdname draw_discrete
 #' @importFrom stats runif
 #' @export
-draw_quantile <- function(type = NULL,
-                          N = NULL) {
+draw_quantile <- function(type, N) {
 
-  if(is.null(N) || !is.numeric(N)) {
-    stop("`N` must be provided to `draw_quantile()` and must be numeric.")
-  }
-  if(!is.null(dim(N)) || length(N) > 1) {
-    stop("`N` must be a single number.")
-  }
-  if(N <= 0) {
-    stop("`N` provided to `draw_quantile()` must be positive.")
+  if(!is_scalar_integerish(N) || N <= 0) {
+    stop("`N` must be provided to `draw_quantile()` and must be a single positive number.")
   }
 
-  if(is.null(type) || !is.numeric(type)) {
-    stop("`type` must be provided to `draw_quantile()` and must be numeric.")
-  }
-  if(!is.null(dim(type)) || length(type) > 1) {
-    stop("`type` must be a single number.")
-  }
-  if(type <= 1) {
-    stop("`type` provided to `draw_quantile()` must be at least 2.")
-  }
-  if(type >= N) {
-    stop("`type` provided to `draw_quantile()` must be less than `N`.")
+  if(!is_scalar_integerish(type) || type <= 1 || type >= N) {
+    stop("`type` must be a single number between 2 and N-1.")
   }
 
   latent_data <- runif(n = N, min = 0, max = 1)
@@ -505,11 +485,11 @@ draw_quantile <- function(type = NULL,
 #' @export
 split_quantile <- function(x = NULL,
                            type = NULL) {
-  if(is.null(x) || length(x) < 2) {
+  if(length(x) < 2) {
     stop("The `x` argument provided to quantile split must be non-null and ",
          "length at least 2.")
   }
-  if(is.null(type) || !is.numeric(type)) {
+  if(!is.numeric(type)) {
     stop("The `type` argument provided to quantile split must be non-null and ",
          "numeric.")
   }
